/**
 * Copyright (c) 2019 Inomera Research.
 * @author Burak Soykal
 */

#import "FNetmera.h"
#import "FNetmeraInboxFilter.h"
#import "FNetmeraPushObject.h"
#import "FNetmeraEvent.h"
#import "FNetmeraUser.h"

@implementation FNetmera

NetmeraInbox  *netmeraInbox;

NSString *const ERROR_CODE_INBOX = @"2016";
NSString *const ERROR_MESSAGE_INBOX = @"There was a problem fetching inbox";
NSString *const ERROR_CODE_NULL_INBOX = @"2017";
NSString *const ERROR_MESSAGE_NULL_INBOX = @"Must call fetchInbox method first";
NSString *const ERROR_CODE_NOT_NEXT_PAGE = @"2018";
NSString *const ERROR_MESSAGE_NOT_NEXT_PAGE = @"Not next page";
NSString *const ERROR_CODE_UPDATE_STATUS = @"2019";
NSString *const ERROR_MESSAGE_UPDATE_STATUS = @"There was a problem updating status";

+ (void)init:(NSString *)apiKey {
    [Netmera start];
    [Netmera setAPIKey:apiKey];
    [self setNetmeraHeaders];
}

+ (void)requestPushNotificationAuthorization {
    [Netmera requestPushNotificationAuthorizationForTypes:UIUserNotificationTypeAlert | UIUserNotificationTypeBadge | UIUserNotificationTypeSound];
}

+ (void)setBaseUrl:(FlutterMethodCall*)call  {
    NSString* baseUrl = call.arguments[@"baseUrl"];
    [Netmera setBaseURL:baseUrl];
}

+ (void)logging: (BOOL) enabled {
    if(enabled){
        [Netmera setLogLevel:(NetmeraLogLevelDebug)];
    }else{
        [Netmera setLogLevel:(NetmeraLogLevelNone)];
    }
}

+ (void)requestPermissionsForLocation {
    [Netmera requestLocationAuthorization];
}

+ (void)setNetmeraMaxActiveRegions: (FlutterMethodCall*)call {
    NSNumber* maxActiveRegions = call.arguments[@"maxActiveRegions"];
    [Netmera setNetmeraMaxActiveRegions:[maxActiveRegions intValue]];
}

+ (void)enablePopupPresentation {
    [Netmera setEnabledPopupPresentation:true];
}

+ (void)disablePopupPresentation {
    [Netmera setEnabledPopupPresentation:false];
}

+ (void)enablePush {
    [Netmera setEnabledReceivingPushNotifications:true];
}

+ (void)disablePush {
    [Netmera setEnabledReceivingPushNotifications:false];
}

+ (void)isPushEnabled: (FlutterResult)result {
    result([NSNumber numberWithBool:[Netmera isEnabledReceivingPushNotifications]]);
}

+ (void)fetchInbox: (FlutterMethodCall*)call result: (FlutterResult)result {
    [Netmera fetchInboxUsingFilter:[FNetmeraInboxFilter getInboxObject: call] completion:^(NetmeraInbox *inbox, NSError *error) {
        [self setInbox: inbox error: error result: result];
    }];
}

+ (void)fetchNextPage:(FlutterResult)result {
    if (netmeraInbox == nil) {
        result([FlutterError errorWithCode:ERROR_CODE_NULL_INBOX message:ERROR_MESSAGE_NULL_INBOX details:nil]);
    }
    if (netmeraInbox.hasNextPage) {
        [netmeraInbox fetchNextPageWithCompletionBlock:^(NSError *error) {
            [self setInbox: netmeraInbox error: error result: result];
        }];
    } else {
        result([FlutterError errorWithCode:ERROR_CODE_NOT_NEXT_PAGE message:ERROR_MESSAGE_NOT_NEXT_PAGE details:nil]);
    }
}

+ (void)setInbox: (NetmeraInbox*)inbox error: (NSError*)error result: (FlutterResult)result {
    if(error) {
        result([FlutterError errorWithCode:ERROR_CODE_INBOX message:ERROR_MESSAGE_INBOX details:nil]);
    } else{
        netmeraInbox = inbox;
        result([FNetmeraPushObject mapPushObjects: inbox.objects]);
    }
}

+ (void)countForStatus: (FlutterMethodCall*) call result:(FlutterResult)result {
    NetmeraInboxStatus status = [call.arguments[@"status"] intValue];
    if (netmeraInbox == nil) {
        result([FlutterError errorWithCode:ERROR_CODE_NULL_INBOX message:ERROR_MESSAGE_NULL_INBOX details:nil]);
    }
    result([NSNumber numberWithInteger:[netmeraInbox countForStatus:status]]);
}

+ (void)inboxUpdateStatus: (FlutterMethodCall*) call result:(FlutterResult)result {
    NSMutableArray* inboxList = call.arguments[@"inboxList"];
    NetmeraInboxStatus status = [call.arguments[@"status"] intValue];
    
    if (netmeraInbox == nil || inboxList == nil) {
        result([FlutterError errorWithCode:ERROR_CODE_NULL_INBOX message:ERROR_MESSAGE_NULL_INBOX details:nil]);
    }
    
    NSMutableArray *selectedInboxList = [[NSMutableArray alloc] init];
    for (int i = 0; i < [inboxList count]; i++) {
        for (int j = 0; j < [netmeraInbox.objects count]; j++) {
            if ([inboxList[i] isEqualToString: netmeraInbox.objects[j].pushId]) {
                [selectedInboxList addObject:netmeraInbox.objects[j]];
            }
        }
    }
    
    [netmeraInbox updateStatus:status
                forPushObjects:selectedInboxList
                    completion:^(NSError *error) {
                        if(error) {
                            result([FlutterError errorWithCode:ERROR_CODE_UPDATE_STATUS message:ERROR_MESSAGE_UPDATE_STATUS details:nil]);
                        }
                    }];
}

+ (void)sendEvent: (FlutterMethodCall*) call {
    NSMutableDictionary* mutableEventDictionary = call.arguments;
    NSArray *keysForNullValues = [mutableEventDictionary allKeysForObject:[NSNull null]];
    [mutableEventDictionary removeObjectsForKeys:keysForNullValues];
    FNetmeraEvent *event = [FNetmeraEvent event];
    event.netmeraEventKey = mutableEventDictionary[@"code"];
    [mutableEventDictionary removeObjectForKey:@"code"];
    event.eventParameters = mutableEventDictionary;
    [Netmera sendEvent:event];
}

+ (void)updateUser: (FlutterMethodCall*) call {
    NSMutableDictionary *userMutableDictionary = call.arguments;
    NSArray *keysForNullValues = [userMutableDictionary allKeysForObject:[NSNull null]];
    [userMutableDictionary removeObjectsForKeys:keysForNullValues];
    
    FNetmeraUser *user = [[FNetmeraUser alloc] init];
    user.userId=[userMutableDictionary objectForKey:@"userId"];
    user.MSISDN=[userMutableDictionary objectForKey:@"msisdn"];
    user.email=[userMutableDictionary objectForKey:@"email"];
    [userMutableDictionary removeObjectForKey:@"userId"];
    [userMutableDictionary removeObjectForKey:@"email"];
    [userMutableDictionary removeObjectForKey:@"msisdn"];
    
    user.userParameters = userMutableDictionary;
    [Netmera updateUser:user];
}

+ (void)setApiKey:(FlutterMethodCall *)call {
    NSString* apiKey = call.arguments[@"apiKey"];
    [Netmera setAPIKey:apiKey];
}

+ (void)handlePushObject:(FlutterMethodCall *)call result:(FlutterResult)result {
    NSString* pushId = call.arguments[@"pushId"];
    
    if (netmeraInbox == nil) {
        result([FlutterError errorWithCode:ERROR_CODE_NULL_INBOX message:ERROR_MESSAGE_NULL_INBOX details:nil]);
    }
    
    for (int i = 0; i < [netmeraInbox.objects count]; i++) {
        if ([pushId isEqualToString: netmeraInbox.objects[i].pushId]) {
            [Netmera handlePushObject:netmeraInbox.objects[i]];
        }
    }
}

+ (void)setNetmeraHeaders {
    [Netmera setFramework:@"flutter"];
    [Netmera setFrameworkVersion:@"1.0.0"];
}

+ (void)setAppGroupName:(FlutterMethodCall *)call {
    NSString* appGroupName = call.arguments[@"appGroupName"];
    [Netmera setAppGroupName:appGroupName];
}

+ (void)turnOffSendingEventAndUserUpdate{
    [Netmera turnOffSendingEventAndUserUpdate];
}

+ (void)setPushDelegate:(NSObject<NetmeraPushDelegate> *)delegate {
    [Netmera setPushDelegate:delegate];
}

@end
