import 'dart:io';

import 'package:flutter/services.dart';
import 'package:netmera_flutter_sdk/events/NetmeraEvent.dart';

import 'NetmeraInboxFilter.dart';
import 'NetmeraPushInbox.dart';
import 'NetmeraUser.dart';

///
/// Copyright (c) 2019 Inomera Research.
/// Author Burak Soykal
///
class Netmera {
  static const _platform = const MethodChannel('netmera_flutter_sdk');

  static const _INIT = "init";
  static const _LOGGING = "logging";
  static const _SET_BASE_URL = "setBaseUrl";
  static const _REQUEST_PERMISSIONS_FOR_LOCATION =
      "requestPermissionsForLocation";
  static const _SET_NETMERA_MAX_ACTIVE_REGIONS = "setNetmeraMaxActiveRegions";
  static const _ENABLE_POPUP_PRESENTATION = "enablePopupPresentation";
  static const _DISABLE_POPUP_PRESENTATION = "disablePopupPresentation";
  static const _ENABLE_PUSH = "enablePush";
  static const _DISABLE_PUSH = "disablePush";
  static const _IS_PUSH_ENABLED = "isPushEnabled";
  static const _TURN_OFF_SENDING_EVENT_AND_USER_UPDATE =
      "turnOffSendingEventAndUserUpdate";
  static const _REQUEST_PUSH_NOTIFICATION_AUTHORIZATION =
      "requestPushNotificationAuthorization";
  static const _FETCH_INBOX = "fetchInbox";
  static const _FETCH_NEXT_PAGE = "fetchNextPage";
  static const _COUNT_FOR_STATUS = "countForStatus";
  static const _INBOX_UPDATE_STATUS = "inboxUpdateStatus";
  static const _SEND_EVENT = "sendEvent";
  static const _UPDATE_USER = "updateUser";
  static const _SET_API_KEY = "setApiKey";
  static const _HANDLE_PUSH_OBJECT = "handlePushObject";
  static const _SET_APP_GROUP_NAME = "setAppGroupName";
  static const _TOGGLE_WEB_VIEW_PROGRESS_BAR_VISIBILITY =
      "toggleWebViewProgressBarVisibility";
  static const _IS_WEB_VIEW_PROGRESS_BAR_ENABLED =
      "isWebViewProgressbarEnabled";

  static const PUSH_OBJECT_STATUS_READ = 1;
  static const PUSH_OBJECT_STATUS_UNREAD = 2;
  static const PUSH_OBJECT_STATUS_DELETED = 4;
  static const PUSH_OBJECT_STATUS_ALL = 7;

  // Netmera Bridge Functions
  static init(String gcmSenderId, String apiKey) {
    Map args = new Map();
    args["apiKey"] = apiKey;

    if (Platform.isAndroid) {
      args["gcmSenderId"] = gcmSenderId;
    }

    _platform.invokeMethod(_INIT, args);
  }

  static logging(bool enabled) {
    _platform.invokeMethod(_LOGGING, {"enabled": enabled});
  }

  static setBaseUrl(String baseUrl) {
    _platform.invokeMethod(_SET_BASE_URL, {"baseUrl": baseUrl});
  }

  static requestPermissionsForLocation() {
    _platform.invokeMethod(_REQUEST_PERMISSIONS_FOR_LOCATION);
  }

  static setNetmeraMaxActiveRegions(int maxActiveRegions) {
    _platform.invokeMethod(_SET_NETMERA_MAX_ACTIVE_REGIONS,
        {"maxActiveRegions": maxActiveRegions});
  }

  static enablePopupPresentation() {
    _platform.invokeMethod(_ENABLE_POPUP_PRESENTATION);
  }

  static disablePopupPresentation() {
    _platform.invokeMethod(_DISABLE_POPUP_PRESENTATION);
  }

  static enablePush() {
    _platform.invokeMethod(_ENABLE_PUSH);
  }

  static disablePush() {
    _platform.invokeMethod(_DISABLE_PUSH);
  }

  static Future<bool> isPushEnabled() async {
    return await _platform.invokeMethod(_IS_PUSH_ENABLED);
  }

  static turnOffSendingEventAndUserUpdate(bool turnOff) {
    _platform.invokeMethod(
        _TURN_OFF_SENDING_EVENT_AND_USER_UPDATE, {"turnOff": turnOff});
  }

  static requestPushNotificationAuthorization() {
    if (Platform.isIOS) {
      _platform.invokeMethod(_REQUEST_PUSH_NOTIFICATION_AUTHORIZATION);
    }
  }

  static Future<List<NetmeraPushInbox>> fetchInbox(
      NetmeraInboxFilter inboxFilter) async {
    try {
      return await _platform
          .invokeMethod(_FETCH_INBOX, inboxFilter.toJson())
          .then((receivedJson) {
        List<NetmeraPushInbox> inboxList = new List();
        receivedJson.forEach((item) {
          inboxList.add(NetmeraPushInbox.fromJson(item));
        });
        return inboxList;
      });
    } catch (e) {
      print(e);
      return new List();
    }
  }

  static Future<List<NetmeraPushInbox>> fetchNextPage() async {
    try {
      return await _platform
          .invokeMethod(_FETCH_NEXT_PAGE)
          .then((receivedJson) {
        List<NetmeraPushInbox> inboxList = new List();
        receivedJson.forEach((item) {
          inboxList.add(NetmeraPushInbox.fromJson(item));
        });
        return inboxList;
      });
    } catch (e) {
      print(e);
      return new List();
    }
  }

  static Future<int> countForStatus(int status) async {
    try {
      return await _platform
          .invokeMethod(_COUNT_FOR_STATUS, {"status": status});
    } catch (e) {
      print(e);
      return -1;
    }
  }

  static inboxUpdateStatus(List<String> inboxList, int status) async {
    try {
      await _platform.invokeMethod(
          _INBOX_UPDATE_STATUS, {"inboxList": inboxList, "status": status});
    } catch (e) {
      print(e);
    }
  }

  static sendEvent(NetmeraEvent event) {
    _platform.invokeMethod(_SEND_EVENT, event.toJson());
  }

  static updateUser(NetmeraUser user) {
    _platform.invokeMethod(_UPDATE_USER, user.toJson());
  }

  static setApiKey(String apiKey) {
    _platform.invokeMethod(_SET_API_KEY, {"apiKey": apiKey});
  }

  static handlePushObject(String pushId) {
    _platform.invokeMethod(_HANDLE_PUSH_OBJECT, {"pushId": pushId});
  }

  static setAppGroupName(String appGroupName) {
    if (Platform.isIOS) {
      _platform
          .invokeMethod(_SET_APP_GROUP_NAME, {"appGroupName": appGroupName});
    }
  }

  static toggleWebViewProgressBarVisibility(bool isVisible) {
    if (Platform.isAndroid) {
      _platform.invokeMethod(
          _TOGGLE_WEB_VIEW_PROGRESS_BAR_VISIBILITY, {"isVisible": isVisible});
    }
  }

  static Future<bool> isWebViewProgressbarEnabled() async {
    if (Platform.isAndroid) {
      return await _platform.invokeMethod(_IS_WEB_VIEW_PROGRESS_BAR_ENABLED);
    }
    return false;
  }
}
